'use strict';

exports.__esModule = true;

var _contentType = require('../../utils/content-type');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var PAGE_CONTENT_TYPE = 'text/html; charset=utf-8';
var EMPTY_PAGE_HTML = '<html><body></body></html>';

var INVALID_BODY_PARAMETER_TYPES = ['number', 'boolean'];

var INVALID_STATUS_CODE_MESSAGE = 'Invalid status code. It should be a number that is greater than 100 and less than 999.';

var ResponseMock = function () {
    function ResponseMock(body, statusCode, headers) {
        _classCallCheck(this, ResponseMock);

        this.body = body;
        this.statusCode = statusCode;
        this.headers = headers;

        this.requestOptions = null;

        this._validateParameters();
    }

    ResponseMock.prototype._validateBody = function _validateBody() {
        var bodyType = typeof this.body;

        if (INVALID_BODY_PARAMETER_TYPES.indexOf(bodyType) !== -1) throw new TypeError(`The 'body' parameter has an invalid type - ${bodyType}.`);
    };

    ResponseMock.prototype._validateStatusCode = function _validateStatusCode() {
        if (this.statusCode === void 0) return;

        if (typeof this.statusCode !== 'number') throw new TypeError(INVALID_STATUS_CODE_MESSAGE);

        var statusCode = parseInt(this.statusCode, 10);

        // NOTE: for Infinity case
        statusCode |= 0;

        if (statusCode < 100 || statusCode > 999) throw new TypeError(INVALID_STATUS_CODE_MESSAGE);
    };

    ResponseMock.prototype._validateHeaders = function _validateHeaders() {
        if (this.headers === void 0) return;

        if (typeof this.headers !== 'object') throw new TypeError('Invalid type of the \'headers\' parameter. It should be an object.');
    };

    ResponseMock.prototype._validateParameters = function _validateParameters() {
        this._validateBody();
        this._validateStatusCode();
        this._validateHeaders();
    };

    ResponseMock.prototype._getContentType = function _getContentType() {
        if (this.body !== null && typeof this.body === 'object') return _contentType.JSON_MIME;

        return PAGE_CONTENT_TYPE;
    };

    ResponseMock.prototype.setRequestOptions = function setRequestOptions(opts) {
        this.requestOptions = opts;
    };

    ResponseMock.prototype.getResponse = function getResponse() {
        var _this = this;

        var response = {
            headers: {
                'content-type': this._getContentType()
            },

            trailers: {},
            statusCode: this.statusCode || 200
        };

        if (this.body === void 0) this.body = EMPTY_PAGE_HTML;else if (typeof this.body === 'function') {
            response.setBody = function (value) {
                _this.body = value;
            };
            response = Object.assign(response, this.body(this.requestOptions, response));
            delete response.setBody;
        }

        if (this.headers) response.headers = Object.assign(response.headers, this.headers);

        return response;
    };

    ResponseMock.prototype.getBody = function getBody() {
        if (!this.body) return new Uint8Array(0);

        var bodyStr = typeof this.body === 'object' ? JSON.stringify(this.body) : String(this.body);

        return Buffer.from(bodyStr);
    };

    return ResponseMock;
}();

exports.default = ResponseMock;
module.exports = exports['default'];